package com.supwisdom.dlpay.framework.controller

import com.supwisdom.dlpay.framework.ResponseBodyBuilder
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import com.supwisdom.dlpay.framework.dao.ApiClientDao
import com.supwisdom.dlpay.framework.domain.AppClientRedis
import com.supwisdom.dlpay.framework.redisrepo.AppClientRepository
import com.supwisdom.dlpay.framework.service.SystemUtilService
import com.supwisdom.dlpay.framework.util.HmacUtil
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.util.*

@RestController
@RequestMapping("/auth")
class ApiAuthController {

    @Autowired
    lateinit var repo: AppClientRepository

    @Autowired
    lateinit var apiClient: ApiClientDao

    @Autowired
    lateinit var systemUtil: SystemUtilService

    @Autowired
    lateinit var jwtUtil: JwtTokenUtil

    @GetMapping("/gettoken")
    fun loginInit(appid: String): ResponseEntity<Any> {
        apiClient.findById(appid).run {
            if (!isPresent) {
                return ResponseEntity.status(401).build()
            }
            if (get().status != "normal") {
                return ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(1, "API状态错误"))
            }
            get()
        }.let {
            val token = generateRandomToken()
            val now = systemUtil.sysdatetime.hostdatetime
            AppClientRedis().apply {
                id = appid
                loginTimestamp = now
                this.token = HmacUtil.HMACSHA256(token, it.secret)
            }.also {
                repo.save(it)
            }
            return ResponseEntity.ok(ResponseBodyBuilder.create()
                    .data("token", token)
                    .data("timestamp", now)
                    .success())
        }
    }

    private fun generateRandomToken(): String {
        val random = ByteArray(12) { 0x00 }
        Random(System.currentTimeMillis()).nextBytes(random)
        return Base64.getEncoder().encode(random).toString(Charsets.UTF_8)
    }

    private fun checkSecretToken(app: AppClientRedis, secret: String): Boolean {
        return (app.token == secret)
    }

    @GetMapping("/authentication")
    fun login(appid: String, secret: String): ResponseEntity<Any> {
        return repo.findById(appid).let {
            if (it.isPresent && checkSecretToken(it.get(), secret)) {
                it.get().also { client ->
                    client.token = ""
                    repo.save(client)
                }
                val token = jwtUtil.generateToken(
                        mapOf("uid" to appid, "issuer" to "supwisdom",
                                "subject" to "payapi"))
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .data("jwt", token)
                        .data("appid", appid)
                        .success())
            } else {
                ResponseEntity.status(401).build()
            }
        }
    }
}